Module: Async::RSpec::Reactor

Defined in:
lib/async/rspec/reactor.rb

Instance Method Summary collapse

Instance Method Details

#notify_failure(exception = $!) ⇒ Object



17
18
19
# File 'lib/async/rspec/reactor.rb', line 17

def notify_failure(exception = $!)
	::RSpec::Support.notify_failure(exception)
end

#run_in_reactor(reactor, duration = nil) ⇒ Object



21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
# File 'lib/async/rspec/reactor.rb', line 21

def run_in_reactor(reactor, duration = nil)
	result = nil
	timer_task = nil
	
	if duration
		timer_task = reactor.async do |task|
			# Wait for the timeout, at any point this task might be cancelled if the user code completes:
			task.annotate("Timer task duration=#{duration}.")
			task.sleep(duration)
			
			# The timeout expired, so generate an error:
			buffer = StringIO.new
			reactor.print_hierarchy(buffer)
			
			# Raise an error so it is logged:
			raise TimeoutError, "Run time exceeded duration #{duration}s:\n#{buffer.string}"
		end
	end
	
	spec_task = reactor.async do |spec_task|
		spec_task.annotate("running example")
		
		result = yield(spec_task)
		
		# We are finished, so stop the timer task if it was started:
		timer_task&.stop
		
		# Now stop the entire reactor:
		raise Async::Stop
	end
	
	begin
		timer_task&.wait
		spec_task.wait
	ensure
		spec_task.stop
	end
	
	return result
end